""" Adapter for vm tool kits
"""
import datetime
import fnmatch
import json
import os
import shutil
import subprocess
import sys

from vik import check
from vik.exception import FileNotFoundError


class Adapter(object):

    def __init__(self, args):
        """ Compose the build arguments into instance variables.

        Args:
            args: original from stripped docopt arguments
        Returns:

        Raises:

        """
        self._attrs = args
        self._attrs.update({'config': 'build.json', 'seq': 'build.seq'})
        tpl = '-'.join((args['release'], args['arch']))
        self._attrs['tpl'] = tpl + '.json'

    def build(self):
        """ Build VM image.
        Build process:
            * copy template
            * mark sequence for build
            * build image.
            * copy image to output location.
            * clean unwanted directory
        """

        # Check the distribution is supported by vik or not.
        if not check.is_supported_distro(self._attrs['distro']):
            sys.exit('The distribution is not yet supported.')

        # Copy the template to the proper location and mark the build sequence.
        self.manipulate_template()
        print('Prepare build configuration...'.ljust(40), '[DONE]')
        # Process to build VM image.
        self.build_image()
        print('Process build image...'.ljust(40), '[DONE]')

        # If it's a locally output capable build, it should output the box
        # image to user specified location.
        targets = ('qemu', 'virtualbox', 'vmware')

        for target in targets:
            if self._attrs['target'] == target:
                output = self._attrs['output']
                to_path = output if output is not None else './'
                self.copy_image(to_path)
                print('Output built image...'.ljust(40), '[DONE]')

        # Clean the left unwanted directories, especially packer cache files.
        if os.path.isdir('packer_cache'):
            shutil.rmtree('packer_cache')

        print('Build VM image...'.ljust(40), '[DONE]')

    def manipulate_template(self):
        """ template manipulate """

        self._vmconfigs_dir = os.path.join(os.path.dirname(__file__),
                                           'vmconfigs')

        # Search template file
        search_tpl_file = self._attrs['tpl']
        copied_tpl_path = os.path.join(self._vmconfigs_dir,
                                       self._attrs['config'])

        try:

            self.__search_and_copy(search_tpl_file, copied_tpl_path)

        except FileNotFoundError:
            sys.exit('vik is missing template file.')

        # modify template file. Change build number(bn) to put the proper
        # template version number(tn) and build date and time(dt) into file.
        if os.path.exists(copied_tpl_path):
            with open(copied_tpl_path, 'r') as build_json:
                build_tpl = json.load(build_json)

            bn = build_tpl['variables']['bn']
            tn = build_tpl['variables']['tn']

            bn = bn.replace('tn', tn)
            bn = bn.replace('dt',
                            datetime.datetime.now().strftime('%Y%m%d-%H%M%S'))
            bn = bn[:-3] + self.__build_sequence()

            build_tpl['variables']['bn'] = bn
            self._attrs['bn'] = bn

            with open(copied_tpl_path, 'w') as build_json:
                json.dump(build_tpl, build_json)

    def build_image(self):
        """ Change into VM configuration directory and run packer build."""

        owd = os.getcwd()

        os.chdir(self._vmconfigs_dir)

        try:
            subprocess.check_call(['packer', 'validate',
                                   self._attrs['config']])
            subprocess.check_call(['packer', 'build', self._attrs['config']])
        except subprocess.CalledProcessError:
            sys.exit()
        finally:
            os.chdir(owd)

    def copy_image(self, to_path):
        """ Copy the output image file to user specified location.

        Args:
            to_path: copy destination.
        Returns:

        Raises:

        """

        search_box_file = self._attrs['bn'] + '*'

        try:

            self.__search_and_copy(search_box_file, to_path)

        except FileNotFoundError:
            sys.exit('built image is not exist, please re-process the build.')

    def __build_sequence(self):
        """ Private method. Build sequence to track build number.
        """

        seq_file = os.path.join(self._vmconfigs_dir, self._attrs['seq'])

        seq = '1'.zfill(3)
        tpl = self._attrs['tpl']
        distro = self._attrs['distro'] + '-' + tpl[:-5]
        first_line = distro + '=' + seq + "\n"

        try:
            if os.stat(seq_file).st_size > 0:
                sequence = open(seq_file, 'r')
                lines = sequence.readlines()
                update = False

                # search for the existed line
                for index, line in enumerate(lines):

                    # if the line existed, increase the build number.
                    if distro in line:
                        idx = line.index('=') + 1
                        pre_seq = line[idx:idx + 3]
                        seq = str(int(pre_seq) + 1).zfill(3)
                        lines[index] = line.replace(pre_seq, seq)
                        update = True
                        break

                # if no line existed, append the new line into file.
                if not update:
                    lines.append(first_line)

                sequence.close()
                sequence = open(seq_file, 'w')
                sequence.writelines(lines)
                return seq
            else:
                sequence = open(seq_file, 'r+')
                sequence.write(first_line)
                return seq
        except OSError:
            sequence = open(seq_file, 'w')
            sequence.write(first_line)
            return seq
        finally:
            sequence.close()

    def __search_and_copy(self, search_pattern, copy_path):
        """ Private method. Search the file then copy the file.

        Args:
            search_pattern: file name pattern.
            copy_path: copy destination.
        Returns:

        Raises:
            FileNotFoundError: if file is not found.
        """

        matched = False

        if os.path.isdir(self._vmconfigs_dir):
            for root, dirs, files in os.walk(self._vmconfigs_dir):
                for file in fnmatch.filter(files, search_pattern):
                    matched = True
                    copy_path = file if copy_path is None else copy_path
                    shutil.copy((os.path.join(root, file)), copy_path)
                    break

        if not matched:
            raise FileNotFoundError('file of search pattern is not found.')
