""" Vm Image Kit

Usage:
    vik build --distro=DISTRO --release=RELEASE
        [--arch=ARCH | --target=PLATFORM | --output=FILE]
    vik clean [--all]
    vik [-hv]

Commands:
    build  build vm image by distribution and corresponding release.
    claen  clean cached installtion medias and cached output images.

Options:
    -a ARCH, --arch=ARCH            OS architectures. [default: amd64]
    -d DISTRO, --distro=DISTRO      OS distribution.
    -r RELEASE, --release=RELEASE   OS release number.
    -t PLATFORM, --target=PLATFORM  Target platform. [default: virtualbox]
    -o FILE, --output=FILE          Output image box.
    --all                           Clean all cached files including downloaded
                                    installation media.
    -h, --help                      Show this help.
    -v, --version                   Show version.

Examples:
    Create debian release 7.4.0 virtualbox image for vagrant use.

        vik build -d debian -r 7.4.0

    Clean internal cached files.

        vik clean

"""
import os
import sys
import shutil

from docopt import docopt

from vik import check
from vik.adapter import Adapter

""" vik use packer for image creation."""
IMAGE_TOOL_CMD = 'packer'


def main():
    """ Main entry point.

    Parse arguments from docopt dictionary and Check the requirements for vik
    processing.

    Args:

    Returns:

    Raises:

    """

    args = docopt(__doc__, version='0.0.2')

    print(args)
    if args['build']:
        # Check if current OS type is suitable for vik processing.
        # If not suitable, raise os not support exception and exit the program.
        if not check.is_os_compatiable():
            sys.exit('system platform is not supported.')

        # Check encapsulates image making tool exists.
        # If not exist, raise tool not found exception and exit the program.
        if not check.has_tool(IMAGE_TOOL_CMD):
            sys.exit('vik reuqire "packer" tool to be installed.')

        print('Requirement test..'.ljust(40), '[DONE]')

        sub_keys = ('--distro', '--release', '--arch', '--target', '--output')
        strip = lambda dict, keys: {key[2:]: dict[key] for key in keys}

        image = Adapter(strip(args, sub_keys))

        image.build()

    elif args['clean']:
        clean(args['--all'])


def clean(all=False):
    """ Clean cahced files.

    Args:
        all: all cached files including the installation media.
    Returns:

    Raises:
    """

    if all:
        packer_cache = _locate('packer_cache')

        if os.path.isdir(packer_cache):
            shutil.rmtree(packer_cache)

    post_processors = _locate('post-processors')

    if os.path.isdir(post_processors):
        for root, dirs, files in os.walk(post_processors):
            for file in files:
                if file != '.gitkeep':
                    os.remove(os.path.join(root, file))
                    print('Clean cached file: ',
                          repr(file).ljust(40),
                          '[DONE]')
    if all:
        print('Clean all cached files and built images.')
    else:
        print('Clean all cached built images.')


def _locate(file_name):
    return os.path.join(os.path.dirname(__file__),
                        'vmconfigs',
                        file_name)

if __name__ == '__main__':
    exit = main()
    if exit:
        sys.exit(exit)
